@@ -1,7 +1,9 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package bond
 
 import (
 	"bufio"
+	_ "embed"
 	"fmt"
 	"os"
 	"path/filepath"
@@ -12,6 +14,10 @@ import (
 	"github.com/influxdata/telegraf/plugins/inputs"
 )
 
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//go:embed sample.conf
+var sampleConfig string
+
 // default host proc path
 const defaultHostProc = "/proc"
 const defaultHostSys = "/sys"
@@ -34,31 +40,7 @@ type sysFiles struct {
 	ADPortsFile string
 }
 
-const sampleConfig = `
-  ## Sets 'proc' directory path
-  ## If not specified, then default is /proc
-  # host_proc = "/proc"
-
-  ## Sets 'sys' directory path
-  ## If not specified, then default is /sys
-  # host_sys = "/sys"
-
-  ## By default, telegraf gather stats for all bond interfaces
-  ## Setting interfaces will restrict the stats to the specified
-  ## bond interfaces.
-  # bond_interfaces = ["bond0"]
-
-  ## Tries to collect additional bond details from /sys/class/net/{bond}
-  ## currently only useful for LACP (mode 4) bonds
-  # collect_sys_details = false
-
-`
-
-func (bond *Bond) Description() string {
-	return "Collect bond interface status, slaves statuses and failures count"
-}
-
-func (bond *Bond) SampleConfig() string {
+func (*Bond) SampleConfig() string {
 	return sampleConfig
 }
 
