@@ -1,6 +1,9 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package cloudwatch
 
 import (
+	"context"
+	_ "embed"
 	"fmt"
 	"net"
 	"net/http"
@@ -9,8 +12,9 @@ import (
 	"sync"
 	"time"
 
-	"github.com/aws/aws-sdk-go/aws"
-	cwClient "github.com/aws/aws-sdk-go/service/cloudwatch"
+	"github.com/aws/aws-sdk-go-v2/aws"
+	cwClient "github.com/aws/aws-sdk-go-v2/service/cloudwatch"
+	"github.com/aws/aws-sdk-go-v2/service/cloudwatch/types"
 
 	"github.com/influxdata/telegraf"
 	"github.com/influxdata/telegraf/config"
@@ -23,6 +27,18 @@ import (
 	"github.com/influxdata/telegraf/plugins/inputs"
 )
 
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//go:embed sample.conf
+var sampleConfig string
+
+const (
+	StatisticAverage     = "Average"
+	StatisticMaximum     = "Maximum"
+	StatisticMinimum     = "Minimum"
+	StatisticSum         = "Sum"
+	StatisticSampleCount = "SampleCount"
+)
+
 // CloudWatch contains the configuration and cache for the cloudwatch plugin.
 type CloudWatch struct {
 	StatisticExclude []string        `toml:"statistic_exclude"`
@@ -72,116 +88,16 @@ type metricCache struct {
 	ttl     time.Duration
 	built   time.Time
 	metrics []filteredMetric
-	queries map[string][]*cwClient.MetricDataQuery
+	queries map[string][]types.MetricDataQuery
 }
 
 type cloudwatchClient interface {
-	ListMetrics(*cwClient.ListMetricsInput) (*cwClient.ListMetricsOutput, error)
-	GetMetricData(*cwClient.GetMetricDataInput) (*cwClient.GetMetricDataOutput, error)
+	ListMetrics(context.Context, *cwClient.ListMetricsInput, ...func(*cwClient.Options)) (*cwClient.ListMetricsOutput, error)
+	GetMetricData(context.Context, *cwClient.GetMetricDataInput, ...func(*cwClient.Options)) (*cwClient.GetMetricDataOutput, error)
 }
 
-// SampleConfig returns the default configuration of the Cloudwatch input plugin.
-func (c *CloudWatch) SampleConfig() string {
-	return `
-  ## Amazon Region
-  region = "us-east-1"
-
-  ## Amazon Credentials
-  ## Credentials are loaded in the following order
-  ## 1) Web identity provider credentials via STS if role_arn and web_identity_token_file are specified
-  ## 2) Assumed credentials via STS if role_arn is specified
-  ## 3) explicit credentials from 'access_key' and 'secret_key'
-  ## 4) shared profile from 'profile'
-  ## 5) environment variables
-  ## 6) shared credentials file
-  ## 7) EC2 Instance Profile
-  # access_key = ""
-  # secret_key = ""
-  # token = ""
-  # role_arn = ""
-  # web_identity_token_file = ""
-  # role_session_name = ""
-  # profile = ""
-  # shared_credential_file = ""
-
-  ## Endpoint to make request against, the correct endpoint is automatically
-  ## determined and this option should only be set if you wish to override the
-  ## default.
-  ##   ex: endpoint_url = "http://localhost:8000"
-  # endpoint_url = ""
-
-  ## Set http_proxy (telegraf uses the system wide proxy settings if it's is not set)
-  # http_proxy_url = "http://localhost:8888"
-
-  # The minimum period for Cloudwatch metrics is 1 minute (60s). However not all
-  # metrics are made available to the 1 minute period. Some are collected at
-  # 3 minute, 5 minute, or larger intervals. See https://aws.amazon.com/cloudwatch/faqs/#monitoring.
-  # Note that if a period is configured that is smaller than the minimum for a
-  # particular metric, that metric will not be returned by the Cloudwatch API
-  # and will not be collected by Telegraf.
-  #
-  ## Requested CloudWatch aggregation Period (required - must be a multiple of 60s)
-  period = "5m"
-
-  ## Collection Delay (required - must account for metrics availability via CloudWatch API)
-  delay = "5m"
-
-  ## Recommended: use metric 'interval' that is a multiple of 'period' to avoid
-  ## gaps or overlap in pulled data
-  interval = "5m"
-
-  ## Recommended if "delay" and "period" are both within 3 hours of request time. Invalid values will be ignored.
-  ## Recently Active feature will only poll for CloudWatch ListMetrics values that occurred within the last 3 Hours.
-  ## If enabled, it will reduce total API usage of the CloudWatch ListMetrics API and require less memory to retain.
-  ## Do not enable if "period" or "delay" is longer than 3 hours, as it will not return data more than 3 hours old.
-  ## See https://docs.aws.amazon.com/AmazonCloudWatch/latest/APIReference/API_ListMetrics.html
-  #recently_active = "PT3H"
-
-  ## Configure the TTL for the internal cache of metrics.
-  # cache_ttl = "1h"
-
-  ## Metric Statistic Namespaces (required)
-  namespaces = ["AWS/ELB"]
-  # A single metric statistic namespace that will be appended to namespaces on startup
-  # namespace = "AWS/ELB"
-
-  ## Maximum requests per second. Note that the global default AWS rate limit is
-  ## 50 reqs/sec, so if you define multiple namespaces, these should add up to a
-  ## maximum of 50.
-  ## See http://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/cloudwatch_limits.html
-  # ratelimit = 25
-
-  ## Timeout for http requests made by the cloudwatch client.
-  # timeout = "5s"
-
-  ## Namespace-wide statistic filters. These allow fewer queries to be made to
-  ## cloudwatch.
-  # statistic_include = [ "average", "sum", "minimum", "maximum", sample_count" ]
-  # statistic_exclude = []
-
-  ## Metrics to Pull
-  ## Defaults to all Metrics in Namespace if nothing is provided
-  ## Refreshes Namespace available metrics every 1h
-  #[[inputs.cloudwatch.metrics]]
-  #  names = ["Latency", "RequestCount"]
-  #
-  #  ## Statistic filters for Metric.  These allow for retrieving specific
-  #  ## statistics for an individual metric.
-  #  # statistic_include = [ "average", "sum", "minimum", "maximum", sample_count" ]
-  #  # statistic_exclude = []
-  #
-  #  ## Dimension filters for Metric.  All dimensions defined for the metric names
-  #  ## must be specified in order to retrieve the metric statistics.
-  #  ## 'value' has wildcard / 'glob' matching support such as 'p-*'.
-  #  [[inputs.cloudwatch.metrics.dimensions]]
-  #    name = "LoadBalancerName"
-  #    value = "p-example"
-`
-}
-
-// Description returns a one-sentence description on the Cloudwatch input plugin.
-func (c *CloudWatch) Description() string {
-	return "Pull Metric Statistics from Amazon CloudWatch"
+func (*CloudWatch) SampleConfig() string {
+	return sampleConfig
 }
 
 func (c *CloudWatch) Init() error {
@@ -227,12 +143,12 @@ func (c *CloudWatch) Gather(acc telegraf.Accumulator) error {
 	wg := sync.WaitGroup{}
 	rLock := sync.Mutex{}
 
-	results := map[string][]*cwClient.MetricDataResult{}
+	results := map[string][]types.MetricDataResult{}
 
 	for namespace, namespacedQueries := range queries {
 		// 500 is the maximum number of metric data queries a `GetMetricData` request can contain.
 		batchSize := 500
-		var batches [][]*cwClient.MetricDataQuery
+		var batches [][]types.MetricDataQuery
 
 		for batchSize < len(namespacedQueries) {
 			namespacedQueries, batches = namespacedQueries[batchSize:], append(batches, namespacedQueries[0:batchSize:batchSize])
@@ -242,7 +158,7 @@ func (c *CloudWatch) Gather(acc telegraf.Accumulator) error {
 		for i := range batches {
 			wg.Add(1)
 			<-lmtr.C
-			go func(n string, inm []*cwClient.MetricDataQuery) {
+			go func(n string, inm []types.MetricDataQuery) {
 				defer wg.Done()
 				result, err := c.gatherMetrics(c.getDataInputs(inm))
 				if err != nil {
@@ -268,8 +184,15 @@ func (c *CloudWatch) initializeCloudWatch() error {
 		return err
 	}
 
-	cfg := &aws.Config{
-		HTTPClient: &http.Client{
+	cfg, err := c.CredentialConfig.Credentials()
+	if err != nil {
+		return err
+	}
+	c.client = cwClient.NewFromConfig(cfg, func(options *cwClient.Options) {
+		// Disable logging
+		options.ClientLogMode = 0
+
+		options.HTTPClient = &http.Client{
 			// use values from DefaultTransport
 			Transport: &http.Transport{
 				Proxy: proxy,
@@ -284,15 +207,8 @@ func (c *CloudWatch) initializeCloudWatch() error {
 				ExpectContinueTimeout: 1 * time.Second,
 			},
 			Timeout: time.Duration(c.Timeout),
-		},
-	}
-
-	loglevel := aws.LogOff
-	p, err := c.CredentialConfig.Credentials()
-	if err != nil {
-		return err
-	}
-	c.client = cwClient.New(p, cfg.WithLogLevel(loglevel))
+		}
+	})
 
 	// Initialize regex matchers for each Dimension value.
 	for _, m := range c.Metrics {
@@ -310,7 +226,7 @@ func (c *CloudWatch) initializeCloudWatch() error {
 }
 
 type filteredMetric struct {
-	metrics    []*cwClient.Metric
+	metrics    []types.Metric
 	statFilter filter.Filter
 }
 
@@ -325,18 +241,18 @@ func getFilteredMetrics(c *CloudWatch) ([]filteredMetric, error) {
 	// check for provided metric filter
 	if c.Metrics != nil {
 		for _, m := range c.Metrics {
-			metrics := []*cwClient.Metric{}
+			metrics := []types.Metric{}
 			if !hasWildcard(m.Dimensions) {
-				dimensions := make([]*cwClient.Dimension, len(m.Dimensions))
+				dimensions := make([]types.Dimension, len(m.Dimensions))
 				for k, d := range m.Dimensions {
-					dimensions[k] = &cwClient.Dimension{
+					dimensions[k] = types.Dimension{
 						Name:  aws.String(d.Name),
 						Value: aws.String(d.Value),
 					}
 				}
 				for _, name := range m.MetricNames {
 					for _, namespace := range c.Namespaces {
-						metrics = append(metrics, &cwClient.Metric{
+						metrics = append(metrics, types.Metric{
 							Namespace:  aws.String(namespace),
 							MetricName: aws.String(name),
 							Dimensions: dimensions,
@@ -352,7 +268,7 @@ func getFilteredMetrics(c *CloudWatch) ([]filteredMetric, error) {
 					for _, metric := range allMetrics {
 						if isSelected(name, metric, m.Dimensions) {
 							for _, namespace := range c.Namespaces {
-								metrics = append(metrics, &cwClient.Metric{
+								metrics = append(metrics, types.Metric{
 									Namespace:  aws.String(namespace),
 									MetricName: aws.String(name),
 									Dimensions: metric.Dimensions,
@@ -385,10 +301,12 @@ func getFilteredMetrics(c *CloudWatch) ([]filteredMetric, error) {
 			return nil, err
 		}
 
-		fMetrics = []filteredMetric{{
-			metrics:    metrics,
-			statFilter: c.statFilter,
-		}}
+		fMetrics = []filteredMetric{
+			{
+				metrics:    metrics,
+				statFilter: c.statFilter,
+			},
+		}
 	}
 
 	c.metricCache = &metricCache{
@@ -401,34 +319,28 @@ func getFilteredMetrics(c *CloudWatch) ([]filteredMetric, error) {
 }
 
 // fetchNamespaceMetrics retrieves available metrics for a given CloudWatch namespace.
-func (c *CloudWatch) fetchNamespaceMetrics() ([]*cwClient.Metric, error) {
-	metrics := []*cwClient.Metric{}
+func (c *CloudWatch) fetchNamespaceMetrics() ([]types.Metric, error) {
+	metrics := []types.Metric{}
 
 	var token *string
-	var params *cwClient.ListMetricsInput
-	var recentlyActive *string
 
-	switch c.RecentlyActive {
-	case "PT3H":
-		recentlyActive = &c.RecentlyActive
-	default:
-		recentlyActive = nil
+	params := &cwClient.ListMetricsInput{
+		Dimensions: []types.DimensionFilter{},
+		NextToken:  token,
+		MetricName: nil,
+	}
+	if c.RecentlyActive == "PT3H" {
+		params.RecentlyActive = types.RecentlyActivePt3h
 	}
 
 	for _, namespace := range c.Namespaces {
-
-		params = &cwClient.ListMetricsInput{
-			Dimensions:     []*cwClient.DimensionFilter{},
-			NextToken:      token,
-			MetricName:     nil,
-			RecentlyActive: recentlyActive,
-			Namespace:      aws.String(namespace),
-		}
-
+		params.Namespace = aws.String(namespace)
 		for {
-			resp, err := c.client.ListMetrics(params)
+			resp, err := c.client.ListMetrics(context.Background(), params)
 			if err != nil {
-				return nil, fmt.Errorf("failed to list metrics with params per namespace: %v", err)
+				c.Log.Errorf("failed to list metrics with namespace %s: %v", namespace, err)
+				// skip problem namespace on error and continue to next namespace
+				break
 			}
 
 			metrics = append(metrics, resp.Metrics...)
@@ -457,75 +369,75 @@ func (c *CloudWatch) updateWindow(relativeTo time.Time) {
 }
 
 // getDataQueries gets all of the possible queries so we can maximize the request payload.
-func (c *CloudWatch) getDataQueries(filteredMetrics []filteredMetric) map[string][]*cwClient.MetricDataQuery {
+func (c *CloudWatch) getDataQueries(filteredMetrics []filteredMetric) map[string][]types.MetricDataQuery {
 	if c.metricCache != nil && c.metricCache.queries != nil && c.metricCache.isValid() {
 		return c.metricCache.queries
 	}
 
 	c.queryDimensions = map[string]*map[string]string{}
 
-	dataQueries := map[string][]*cwClient.MetricDataQuery{}
+	dataQueries := map[string][]types.MetricDataQuery{}
 	for i, filtered := range filteredMetrics {
 		for j, metric := range filtered.metrics {
 			id := strconv.Itoa(j) + "_" + strconv.Itoa(i)
 			dimension := ctod(metric.Dimensions)
 			if filtered.statFilter.Match("average") {
 				c.queryDimensions["average_"+id] = dimension
-				dataQueries[*metric.Namespace] = append(dataQueries[*metric.Namespace], &cwClient.MetricDataQuery{
+				dataQueries[*metric.Namespace] = append(dataQueries[*metric.Namespace], types.MetricDataQuery{
 					Id:    aws.String("average_" + id),
 					Label: aws.String(snakeCase(*metric.MetricName + "_average")),
-					MetricStat: &cwClient.MetricStat{
-						Metric: metric,
-						Period: aws.Int64(int64(time.Duration(c.Period).Seconds())),
-						Stat:   aws.String(cwClient.StatisticAverage),
+					MetricStat: &types.MetricStat{
+						Metric: &filtered.metrics[j],
+						Period: aws.Int32(int32(time.Duration(c.Period).Seconds())),
+						Stat:   aws.String(StatisticAverage),
 					},
 				})
 			}
 			if filtered.statFilter.Match("maximum") {
 				c.queryDimensions["maximum_"+id] = dimension
-				dataQueries[*metric.Namespace] = append(dataQueries[*metric.Namespace], &cwClient.MetricDataQuery{
+				dataQueries[*metric.Namespace] = append(dataQueries[*metric.Namespace], types.MetricDataQuery{
 					Id:    aws.String("maximum_" + id),
 					Label: aws.String(snakeCase(*metric.MetricName + "_maximum")),
-					MetricStat: &cwClient.MetricStat{
-						Metric: metric,
-						Period: aws.Int64(int64(time.Duration(c.Period).Seconds())),
-						Stat:   aws.String(cwClient.StatisticMaximum),
+					MetricStat: &types.MetricStat{
+						Metric: &filtered.metrics[j],
+						Period: aws.Int32(int32(time.Duration(c.Period).Seconds())),
+						Stat:   aws.String(StatisticMaximum),
 					},
 				})
 			}
 			if filtered.statFilter.Match("minimum") {
 				c.queryDimensions["minimum_"+id] = dimension
-				dataQueries[*metric.Namespace] = append(dataQueries[*metric.Namespace], &cwClient.MetricDataQuery{
+				dataQueries[*metric.Namespace] = append(dataQueries[*metric.Namespace], types.MetricDataQuery{
 					Id:    aws.String("minimum_" + id),
 					Label: aws.String(snakeCase(*metric.MetricName + "_minimum")),
-					MetricStat: &cwClient.MetricStat{
-						Metric: metric,
-						Period: aws.Int64(int64(time.Duration(c.Period).Seconds())),
-						Stat:   aws.String(cwClient.StatisticMinimum),
+					MetricStat: &types.MetricStat{
+						Metric: &filtered.metrics[j],
+						Period: aws.Int32(int32(time.Duration(c.Period).Seconds())),
+						Stat:   aws.String(StatisticMinimum),
 					},
 				})
 			}
 			if filtered.statFilter.Match("sum") {
 				c.queryDimensions["sum_"+id] = dimension
-				dataQueries[*metric.Namespace] = append(dataQueries[*metric.Namespace], &cwClient.MetricDataQuery{
+				dataQueries[*metric.Namespace] = append(dataQueries[*metric.Namespace], types.MetricDataQuery{
 					Id:    aws.String("sum_" + id),
 					Label: aws.String(snakeCase(*metric.MetricName + "_sum")),
-					MetricStat: &cwClient.MetricStat{
-						Metric: metric,
-						Period: aws.Int64(int64(time.Duration(c.Period).Seconds())),
-						Stat:   aws.String(cwClient.StatisticSum),
+					MetricStat: &types.MetricStat{
+						Metric: &filtered.metrics[j],
+						Period: aws.Int32(int32(time.Duration(c.Period).Seconds())),
+						Stat:   aws.String(StatisticSum),
 					},
 				})
 			}
 			if filtered.statFilter.Match("sample_count") {
 				c.queryDimensions["sample_count_"+id] = dimension
-				dataQueries[*metric.Namespace] = append(dataQueries[*metric.Namespace], &cwClient.MetricDataQuery{
+				dataQueries[*metric.Namespace] = append(dataQueries[*metric.Namespace], types.MetricDataQuery{
 					Id:    aws.String("sample_count_" + id),
 					Label: aws.String(snakeCase(*metric.MetricName + "_sample_count")),
-					MetricStat: &cwClient.MetricStat{
-						Metric: metric,
-						Period: aws.Int64(int64(time.Duration(c.Period).Seconds())),
-						Stat:   aws.String(cwClient.StatisticSampleCount),
+					MetricStat: &types.MetricStat{
+						Metric: &filtered.metrics[j],
+						Period: aws.Int32(int32(time.Duration(c.Period).Seconds())),
+						Stat:   aws.String(StatisticSampleCount),
 					},
 				})
 			}
@@ -553,11 +465,11 @@ func (c *CloudWatch) getDataQueries(filteredMetrics []filteredMetric) map[string
 // gatherMetrics gets metric data from Cloudwatch.
 func (c *CloudWatch) gatherMetrics(
 	params *cwClient.GetMetricDataInput,
-) ([]*cwClient.MetricDataResult, error) {
-	results := []*cwClient.MetricDataResult{}
+) ([]types.MetricDataResult, error) {
+	results := []types.MetricDataResult{}
 
 	for {
-		resp, err := c.client.GetMetricData(params)
+		resp, err := c.client.GetMetricData(context.Background(), params)
 		if err != nil {
 			return nil, fmt.Errorf("failed to get metric data: %v", err)
 		}
@@ -574,7 +486,7 @@ func (c *CloudWatch) gatherMetrics(
 
 func (c *CloudWatch) aggregateMetrics(
 	acc telegraf.Accumulator,
-	metricDataResults map[string][]*cwClient.MetricDataResult,
+	metricDataResults map[string][]types.MetricDataResult,
 ) error {
 	var (
 		grouper = internalMetric.NewSeriesGrouper()
@@ -592,7 +504,7 @@ func (c *CloudWatch) aggregateMetrics(
 			tags["region"] = c.Region
 
 			for i := range result.Values {
-				if err := grouper.Add(namespace, tags, *result.Timestamps[i], *result.Label, *result.Values[i]); err != nil {
+				if err := grouper.Add(namespace, tags, result.Timestamps[i], *result.Label, result.Values[i]); err != nil {
 					acc.AddError(err)
 				}
 			}
@@ -622,20 +534,20 @@ func New() *CloudWatch {
 }
 
 func sanitizeMeasurement(namespace string) string {
-	namespace = strings.Replace(namespace, "/", "_", -1)
+	namespace = strings.ReplaceAll(namespace, "/", "_")
 	namespace = snakeCase(namespace)
 	return "cloudwatch_" + namespace
 }
 
 func snakeCase(s string) string {
 	s = internal.SnakeCase(s)
-	s = strings.Replace(s, " ", "_", -1)
-	s = strings.Replace(s, "__", "_", -1)
+	s = strings.ReplaceAll(s, " ", "_")
+	s = strings.ReplaceAll(s, "__", "_")
 	return s
 }
 
 // ctod converts cloudwatch dimensions to regular dimensions.
-func ctod(cDimensions []*cwClient.Dimension) *map[string]string {
+func ctod(cDimensions []types.Dimension) *map[string]string {
 	dimensions := map[string]string{}
 	for i := range cDimensions {
 		dimensions[snakeCase(*cDimensions[i].Name)] = *cDimensions[i].Value
@@ -643,7 +555,7 @@ func ctod(cDimensions []*cwClient.Dimension) *map[string]string {
 	return &dimensions
 }
 
-func (c *CloudWatch) getDataInputs(dataQueries []*cwClient.MetricDataQuery) *cwClient.GetMetricDataInput {
+func (c *CloudWatch) getDataInputs(dataQueries []types.MetricDataQuery) *cwClient.GetMetricDataInput {
 	return &cwClient.GetMetricDataInput{
 		StartTime:         aws.Time(c.windowStart),
 		EndTime:           aws.Time(c.windowEnd),
@@ -665,7 +577,7 @@ func hasWildcard(dimensions []*Dimension) bool {
 	return false
 }
 
-func isSelected(name string, metric *cwClient.Metric, dimensions []*Dimension) bool {
+func isSelected(name string, metric types.Metric, dimensions []*Dimension) bool {
 	if name != *metric.MetricName {
 		return false
 	}
