@@ -1,54 +1,61 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package internet_speed
 
 import (
+	_ "embed"
 	"fmt"
+	"time"
+
+	"github.com/showwin/speedtest-go/speedtest"
 
 	"github.com/influxdata/telegraf"
 	"github.com/influxdata/telegraf/plugins/inputs"
-	"github.com/showwin/speedtest-go/speedtest"
 )
 
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//go:embed sample.conf
+var sampleConfig string
+
 // InternetSpeed is used to store configuration values.
 type InternetSpeed struct {
 	EnableFileDownload bool            `toml:"enable_file_download"`
+	Cache              bool            `toml:"cache"`
 	Log                telegraf.Logger `toml:"-"`
-}
-
-const sampleConfig = `
-  ## Sets if runs file download test
-  ## Default: false  
-  enable_file_download = false
-`
-
-// Description returns information about the plugin.
-func (is *InternetSpeed) Description() string {
-	return "Monitors internet speed using speedtest.net service"
-}
-
-// SampleConfig displays configuration instructions.
-func (is *InternetSpeed) SampleConfig() string {
-	return sampleConfig
+	serverCache        *speedtest.Server
 }
 
 const measurement = "internet_speed"
 
+func (*InternetSpeed) SampleConfig() string {
+	return sampleConfig
+}
+
 func (is *InternetSpeed) Gather(acc telegraf.Accumulator) error {
-	user, err := speedtest.FetchUserInfo()
-	if err != nil {
-		return fmt.Errorf("fetching user info failed: %v", err)
-	}
-	serverList, err := speedtest.FetchServerList(user)
-	if err != nil {
-		return fmt.Errorf("fetching server list failed: %v", err)
+
+	// Get closest server
+	s := is.serverCache
+	if s == nil {
+		user, err := speedtest.FetchUserInfo()
+		if err != nil {
+			return fmt.Errorf("fetching user info failed: %v", err)
+		}
+		serverList, err := speedtest.FetchServers(user)
+		if err != nil {
+			return fmt.Errorf("fetching server list failed: %v", err)
+		}
+		if len(serverList) < 1 {
+			return fmt.Errorf("no servers found")
+		}
+		s = serverList[0]
+		is.Log.Debugf("Found server: %v", s)
+		if is.Cache {
+			is.serverCache = s
+		}
 	}
 
-	if len(serverList.Servers) < 1 {
-		return fmt.Errorf("no servers found")
-	}
-	s := serverList.Servers[0]
 	is.Log.Debug("Starting Speed Test")
 	is.Log.Debug("Running Ping...")
-	err = s.PingTest()
+	err := s.PingTest()
 	if err != nil {
 		return fmt.Errorf("ping test failed: %v", err)
 	}
@@ -68,15 +75,20 @@ func (is *InternetSpeed) Gather(acc telegraf.Accumulator) error {
 	fields := make(map[string]interface{})
 	fields["download"] = s.DLSpeed
 	fields["upload"] = s.ULSpeed
-	fields["latency"] = s.Latency
+	fields["latency"] = timeDurationMillisecondToFloat64(s.Latency)
 
 	tags := make(map[string]string)
 
 	acc.AddFields(measurement, fields, tags)
 	return nil
 }
+
 func init() {
 	inputs.Add("internet_speed", func() telegraf.Input {
 		return &InternetSpeed{}
 	})
 }
+
+func timeDurationMillisecondToFloat64(d time.Duration) float64 {
+	return float64(d) / float64(time.Millisecond)
+}
