@@ -1,8 +1,8 @@
-//go:build !openbsd
-
+//go:generate ../../../tools/readme_config_includer/generator
 package modbus
 
 import (
+	_ "embed"
 	"fmt"
 	"net"
 	"net/url"
@@ -17,6 +17,14 @@ import (
 	"github.com/influxdata/telegraf/plugins/inputs"
 )
 
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//go:embed sample_general_begin.conf
+var sampleConfigStart string
+
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//go:embed sample_general_end.conf
+var sampleConfigEnd string
+
 type ModbusWorkarounds struct {
 	PollPause        config.Duration `toml:"pause_between_requests"`
 	CloseAfterGather bool            `toml:"close_connection_after_gather"`
@@ -38,7 +46,10 @@ type Modbus struct {
 	Workarounds      ModbusWorkarounds `toml:"workarounds"`
 	Log              telegraf.Logger   `toml:"-"`
 	// Register configuration
+	ConfigurationType string `toml:"configuration_type"`
 	ConfigurationOriginal
+	ConfigurationPerRequest
+
 	// Connection handling
 	client      mb.Client
 	handler     mb.ClientHandler
@@ -59,9 +70,9 @@ type requestSet struct {
 type field struct {
 	measurement string
 	name        string
-	scale       float64
 	address     uint16
 	length      uint16
+	omit        bool
 	converter   fieldConverterFunc
 	value       interface{}
 }
@@ -73,114 +84,20 @@ const (
 	cInputRegisters   = "input_register"
 )
 
-const description = `Retrieve data from MODBUS slave devices`
-const sampleConfig = `
-  ## Connection Configuration
-  ##
-  ## The plugin supports connections to PLCs via MODBUS/TCP, RTU over TCP, ASCII over TCP or
-  ## via serial line communication in binary (RTU) or readable (ASCII) encoding
-  ##
-  ## Device name
-  name = "Device"
-
-  ## Slave ID - addresses a MODBUS device on the bus
-  ## Range: 0 - 255 [0 = broadcast; 248 - 255 = reserved]
-  slave_id = 1
-
-  ## Timeout for each request
-  timeout = "1s"
-
-  ## Maximum number of retries and the time to wait between retries
-  ## when a slave-device is busy.
-  # busy_retries = 0
-  # busy_retries_wait = "100ms"
-
-  # TCP - connect via Modbus/TCP
-  controller = "tcp://localhost:502"
-
-  ## Serial (RS485; RS232)
-  # controller = "file:///dev/ttyUSB0"
-  # baud_rate = 9600
-  # data_bits = 8
-  # parity = "N"
-  # stop_bits = 1
-  # transmission_mode = "RTU"
-
-  ## Trace the connection to the modbus device as debug messages
-  ## Note: You have to enable telegraf's debug mode to see those messages!
-  # debug_connection = false
-
-  ## For Modbus over TCP you can choose between "TCP", "RTUoverTCP" and "ASCIIoverTCP"
-  ## default behaviour is "TCP" if the controller is TCP
-  ## For Serial you can choose between "RTU" and "ASCII"
-  # transmission_mode = "RTU"
-
-  ## Measurements
-  ##
-
-  ## Digital Variables, Discrete Inputs and Coils
-  ## measurement - the (optional) measurement name, defaults to "modbus"
-  ## name        - the variable name
-  ## address     - variable address
-
-  discrete_inputs = [
-    { name = "start",          address = [0]},
-    { name = "stop",           address = [1]},
-    { name = "reset",          address = [2]},
-    { name = "emergency_stop", address = [3]},
-  ]
-  coils = [
-    { name = "motor1_run",     address = [0]},
-    { name = "motor1_jog",     address = [1]},
-    { name = "motor1_stop",    address = [2]},
-  ]
-
-  ## Analog Variables, Input Registers and Holding Registers
-  ## measurement - the (optional) measurement name, defaults to "modbus"
-  ## name        - the variable name
-  ## byte_order  - the ordering of bytes
-  ##  |---AB, ABCD   - Big Endian
-  ##  |---BA, DCBA   - Little Endian
-  ##  |---BADC       - Mid-Big Endian
-  ##  |---CDAB       - Mid-Little Endian
-  ## data_type  - INT16, UINT16, INT32, UINT32, INT64, UINT64,
-  ##              FLOAT32-IEEE, FLOAT64-IEEE (the IEEE 754 binary representation)
-  ##              FLOAT32, FIXED, UFIXED (fixed-point representation on input)
-  ## scale      - the final numeric variable representation
-  ## address    - variable address
-
-  holding_registers = [
-    { name = "power_factor", byte_order = "AB",   data_type = "FIXED", scale=0.01,  address = [8]},
-    { name = "voltage",      byte_order = "AB",   data_type = "FIXED", scale=0.1,   address = [0]},
-    { name = "energy",       byte_order = "ABCD", data_type = "FIXED", scale=0.001, address = [5,6]},
-    { name = "current",      byte_order = "ABCD", data_type = "FIXED", scale=0.001, address = [1,2]},
-    { name = "frequency",    byte_order = "AB",   data_type = "UFIXED", scale=0.1,  address = [7]},
-    { name = "power",        byte_order = "ABCD", data_type = "UFIXED", scale=0.1,  address = [3,4]},
-  ]
-  input_registers = [
-    { name = "tank_level",   byte_order = "AB",   data_type = "INT16",   scale=1.0,     address = [0]},
-    { name = "tank_ph",      byte_order = "AB",   data_type = "INT16",   scale=1.0,     address = [1]},
-    { name = "pump1_speed",  byte_order = "ABCD", data_type = "INT32",   scale=1.0,     address = [3,4]},
-  ]
-
-  ## Enable workarounds required by some devices to work correctly
-  # [inputs.modbus.workarounds]
-    ## Pause between read requests sent to the device. This might be necessary for (slow) serial devices.
-    # pause_between_requests = "0ms"
-    ## Close the connection after every gather cycle. Usually the plugin closes the connection after a certain
-    ## idle-timeout, however, if you query a device with limited simultaneous connectivity (e.g. serial devices)
-    ## from multiple instances you might want to only stay connected during gather and disconnect afterwards.
-    # close_connection_after_gather = false
-`
-
 // SampleConfig returns a basic configuration for the plugin
 func (m *Modbus) SampleConfig() string {
-	return sampleConfig
-}
+	configs := []Configuration{}
+	cfgOriginal := m.ConfigurationOriginal
+	cfgPerRequest := m.ConfigurationPerRequest
+	configs = append(configs, &cfgOriginal, &cfgPerRequest)
 
-// Description returns a short description of what the plugin does
-func (m *Modbus) Description() string {
-	return description
+	totalConfig := sampleConfigStart
+	for _, c := range configs {
+		totalConfig += c.SampleConfigPart() + "\n"
+	}
+	totalConfig += "\n"
+	totalConfig += sampleConfigEnd
+	return totalConfig
 }
 
 func (m *Modbus) Init() error {
@@ -193,14 +110,25 @@ func (m *Modbus) Init() error {
 		return fmt.Errorf("retries cannot be negative")
 	}
 
-	// Check and process the configuration
-	if err := m.ConfigurationOriginal.Check(); err != nil {
-		return fmt.Errorf("original configuraton invalid: %v", err)
+	// Determine the configuration style
+	var cfg Configuration
+	switch m.ConfigurationType {
+	case "", "register":
+		cfg = &m.ConfigurationOriginal
+	case "request":
+		cfg = &m.ConfigurationPerRequest
+	default:
+		return fmt.Errorf("unknown configuration type %q", m.ConfigurationType)
 	}
 
-	r, err := m.ConfigurationOriginal.Process()
+	// Check and process the configuration
+	if err := cfg.Check(); err != nil {
+		return fmt.Errorf("configuraton invalid: %v", err)
+	}
+
+	r, err := cfg.Process()
 	if err != nil {
-		return fmt.Errorf("cannot process original configuraton: %v", err)
+		return fmt.Errorf("cannot process configuraton: %v", err)
 	}
 	m.requests = r
 
@@ -331,7 +259,6 @@ func (m *Modbus) initClient() error {
 		return fmt.Errorf("invalid controller %q", m.Controller)
 	}
 
-	m.handler.SetSlave(m.SlaveID)
 	m.client = mb.NewClient(m.handler)
 	m.isConnected = false
 
@@ -352,7 +279,8 @@ func (m *Modbus) disconnect() error {
 }
 
 func (m *Modbus) gatherFields() error {
-	for _, requests := range m.requests {
+	for slaveID, requests := range m.requests {
+		m.handler.SetSlave(slaveID)
 		if err := m.gatherRequestsCoil(requests.coil); err != nil {
 			return err
 		}
@@ -479,6 +407,15 @@ func (m *Modbus) gatherRequestsInput(requests []request) error {
 func (m *Modbus) collectFields(acc telegraf.Accumulator, timestamp time.Time, tags map[string]string, requests []request) {
 	grouper := metric.NewSeriesGrouper()
 	for _, request := range requests {
+		// Collect tags from global and per-request
+		rtags := map[string]string{}
+		for k, v := range tags {
+			rtags[k] = v
+		}
+		for k, v := range request.tags {
+			rtags[k] = v
+		}
+
 		for _, field := range request.fields {
 			// In case no measurement was specified we use "modbus" as default
 			measurement := "modbus"
@@ -487,7 +424,7 @@ func (m *Modbus) collectFields(acc telegraf.Accumulator, timestamp time.Time, ta
 			}
 
 			// Group the data by series
-			if err := grouper.Add(measurement, tags, timestamp, field.name, field.value); err != nil {
+			if err := grouper.Add(measurement, rtags, timestamp, field.name, field.value); err != nil {
 				acc.AddError(fmt.Errorf("cannot add field %q for measurement %q: %v", field.name, measurement, err))
 				continue
 			}
