@@ -1,7 +1,9 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package smart
 
 import (
 	"bufio"
+	_ "embed"
 	"fmt"
 	"os"
 	"os/exec"
@@ -19,6 +21,10 @@ import (
 	"github.com/influxdata/telegraf/plugins/inputs"
 )
 
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//go:embed sample.conf
+var sampleConfig string
+
 const intelVID = "0x8086"
 
 var (
@@ -347,56 +353,6 @@ type nvmeDevice struct {
 	serialNumber string
 }
 
-var sampleConfig = `
-  ## Optionally specify the path to the smartctl executable
-  # path_smartctl = "/usr/bin/smartctl"
-
-  ## Optionally specify the path to the nvme-cli executable
-  # path_nvme = "/usr/bin/nvme"
-
-  ## Optionally specify if vendor specific attributes should be propagated for NVMe disk case
-  ## ["auto-on"] - automatically find and enable additional vendor specific disk info
-  ## ["vendor1", "vendor2", ...] - e.g. "Intel" enable additional Intel specific disk info
-  # enable_extensions = ["auto-on"]
-
-  ## On most platforms used cli utilities requires root access.
-  ## Setting 'use_sudo' to true will make use of sudo to run smartctl or nvme-cli.
-  ## Sudo must be configured to allow the telegraf user to run smartctl or nvme-cli
-  ## without a password.
-  # use_sudo = false
-
-  ## Skip checking disks in this power mode. Defaults to
-  ## "standby" to not wake up disks that have stopped rotating.
-  ## See --nocheck in the man pages for smartctl.
-  ## smartctl version 5.41 and 5.42 have faulty detection of
-  ## power mode and might require changing this value to
-  ## "never" depending on your disks.
-  # nocheck = "standby"
-
-  ## Gather all returned S.M.A.R.T. attribute metrics and the detailed
-  ## information from each drive into the 'smart_attribute' measurement.
-  # attributes = false
-
-  ## Optionally specify devices to exclude from reporting if disks auto-discovery is performed.
-  # excludes = [ "/dev/pass6" ]
-
-  ## Optionally specify devices and device type, if unset
-  ## a scan (smartctl --scan and smartctl --scan -d nvme) for S.M.A.R.T. devices will be done
-  ## and all found will be included except for the excluded in excludes.
-  # devices = [ "/dev/ada0 -d atacam", "/dev/nvme0"]
-
-  ## Timeout for the cli command to complete.
-  # timeout = "30s"
-
-  ## Optionally call smartctl and nvme-cli with a specific concurrency policy.
-  ## By default, smartctl and nvme-cli are called in separate threads (goroutines) to gather disk attributes.
-  ## Some devices (e.g. disks in RAID arrays) may have access limitations that require sequential reading of
-  ## SMART data - one individual array drive at the time. In such case please set this configuration option
-  ## to "sequential" to get readings for all drives.
-  ## valid options: concurrent, sequential
-  # read_method = "concurrent"
-`
-
 func newSmart() *Smart {
 	return &Smart{
 		Timeout:    config.Duration(time.Second * 30),
@@ -404,16 +360,10 @@ func newSmart() *Smart {
 	}
 }
 
-// SampleConfig returns sample configuration for this plugin.
-func (m *Smart) SampleConfig() string {
+func (*Smart) SampleConfig() string {
 	return sampleConfig
 }
 
-// Description returns the plugin description.
-func (m *Smart) Description() string {
-	return "Read metrics from storage devices supporting S.M.A.R.T."
-}
-
 // Init performs one time setup of the plugin and returns an error if the configuration is invalid.
 func (m *Smart) Init() error {
 	//if deprecated `path` (to smartctl binary) is provided in config and `path_smartctl` override does not exist
@@ -790,12 +740,12 @@ func (m *Smart) gatherDisk(acc telegraf.Accumulator, device string, wg *sync.Wai
 
 		wwn := wwnInfo.FindStringSubmatch(line)
 		if len(wwn) > 1 {
-			deviceTags["wwn"] = strings.Replace(wwn[1], " ", "", -1)
+			deviceTags["wwn"] = strings.ReplaceAll(wwn[1], " ", "")
 		}
 
 		capacity := userCapacityInfo.FindStringSubmatch(line)
 		if len(capacity) > 1 {
-			deviceTags["capacity"] = strings.Replace(capacity[1], ",", "", -1)
+			deviceTags["capacity"] = strings.ReplaceAll(capacity[1], ",", "")
 		}
 
 		enabled := smartEnabledInfo.FindStringSubmatch(line)
@@ -1064,7 +1014,7 @@ func parseDataUnits(fields, deviceFields map[string]interface{}, str string) err
 }
 
 func parseCommaSeparatedIntWithAccumulator(acc telegraf.Accumulator, fields map[string]interface{}, tags map[string]string, str string) error {
-	i, err := strconv.ParseInt(strings.Replace(str, ",", "", -1), 10, 64)
+	i, err := strconv.ParseInt(strings.ReplaceAll(str, ",", ""), 10, 64)
 	if err != nil {
 		return err
 	}
