@@ -1,7 +1,9 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package amon
 
 import (
 	"bytes"
+	_ "embed"
 	"encoding/json"
 	"fmt"
 	"net/http"
@@ -13,6 +15,10 @@ import (
 	"github.com/influxdata/telegraf/plugins/outputs"
 )
 
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//go:embed sample.conf
+var sampleConfig string
+
 type Amon struct {
 	ServerKey    string          `toml:"server_key"`
 	AmonInstance string          `toml:"amon_instance"`
@@ -22,17 +28,6 @@ type Amon struct {
 	client *http.Client
 }
 
-var sampleConfig = `
-  ## Amon Server Key
-  server_key = "my-server-key" # required.
-
-  ## Amon Instance URL
-  amon_instance = "https://youramoninstance" # required
-
-  ## Connection timeout.
-  # timeout = "5s"
-`
-
 type TimeSeries struct {
 	Series []*Metric `json:"series"`
 }
@@ -44,6 +39,10 @@ type Metric struct {
 
 type Point [2]float64
 
+func (*Amon) SampleConfig() string {
+	return sampleConfig
+}
+
 func (a *Amon) Connect() error {
 	if a.ServerKey == "" || a.AmonInstance == "" {
 		return fmt.Errorf("serverkey and amon_instance are required fields for amon output")
@@ -66,11 +65,11 @@ func (a *Amon) Write(metrics []telegraf.Metric) error {
 	metricCounter := 0
 
 	for _, m := range metrics {
-		mname := strings.Replace(m.Name(), "_", ".", -1)
+		mname := strings.ReplaceAll(m.Name(), "_", ".")
 		if amonPts, err := buildMetrics(m); err == nil {
 			for fieldName, amonPt := range amonPts {
 				metric := &Metric{
-					Metric: mname + "_" + strings.Replace(fieldName, "_", ".", -1),
+					Metric: mname + "_" + strings.ReplaceAll(fieldName, "_", "."),
 				}
 				metric.Points[0] = amonPt
 				tempSeries = append(tempSeries, metric)
@@ -106,14 +105,6 @@ func (a *Amon) Write(metrics []telegraf.Metric) error {
 	return nil
 }
 
-func (a *Amon) SampleConfig() string {
-	return sampleConfig
-}
-
-func (a *Amon) Description() string {
-	return "Configuration for Amon Server to send metrics to."
-}
-
 func (a *Amon) authenticatedURL() string {
 	return fmt.Sprintf("%s/api/system/%s", a.AmonInstance, a.ServerKey)
 }
@@ -142,7 +133,7 @@ func (p *Point) setValue(v interface{}) error {
 	case float32:
 		p[1] = float64(d)
 	case float64:
-		p[1] = float64(d)
+		p[1] = d
 	default:
 		return fmt.Errorf("undeterminable type")
 	}
