@@ -1,11 +1,13 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package amqp
 
 import (
 	"bytes"
+	_ "embed"
 	"strings"
 	"time"
 
-	"github.com/streadway/amqp"
+	amqp "github.com/rabbitmq/amqp091-go"
 
 	"github.com/influxdata/telegraf"
 	"github.com/influxdata/telegraf/config"
@@ -15,6 +17,10 @@ import (
 	"github.com/influxdata/telegraf/plugins/serializers"
 )
 
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//go:embed sample.conf
+var sampleConfig string
+
 const (
 	DefaultURL             = "amqp://localhost:5672/influxdb"
 	DefaultAuthMethod      = "PLAIN"
@@ -34,7 +40,7 @@ func (a *externalAuth) Response() string {
 }
 
 type AMQP struct {
-	URL                string            `toml:"url"` // deprecated in 1.7; use brokers
+	URL                string            `toml:"url" deprecated:"1.7.0;use 'brokers' instead"`
 	Brokers            []string          `toml:"brokers"`
 	Exchange           string            `toml:"exchange"`
 	ExchangeType       string            `toml:"exchange_type"`
@@ -48,9 +54,9 @@ type AMQP struct {
 	RoutingTag         string            `toml:"routing_tag"`
 	RoutingKey         string            `toml:"routing_key"`
 	DeliveryMode       string            `toml:"delivery_mode"`
-	Database           string            `toml:"database"`         // deprecated in 1.7; use headers
-	RetentionPolicy    string            `toml:"retention_policy"` // deprecated in 1.7; use headers
-	Precision          string            `toml:"precision"`        // deprecated; has no effect
+	Database           string            `toml:"database" deprecated:"1.7.0;use 'headers' instead"`
+	RetentionPolicy    string            `toml:"retention_policy" deprecated:"1.7.0;use 'headers' instead"`
+	Precision          string            `toml:"precision" deprecated:"1.2.0;option is ignored"`
 	Headers            map[string]string `toml:"headers"`
 	Timeout            config.Duration   `toml:"timeout"`
 	UseBatchFormat     bool              `toml:"use_batch_format"`
@@ -71,110 +77,10 @@ type Client interface {
 	Close() error
 }
 
-var sampleConfig = `
-  ## Broker to publish to.
-  ##   deprecated in 1.7; use the brokers option
-  # url = "amqp://localhost:5672/influxdb"
-
-  ## Brokers to publish to.  If multiple brokers are specified a random broker
-  ## will be selected anytime a connection is established.  This can be
-  ## helpful for load balancing when not using a dedicated load balancer.
-  brokers = ["amqp://localhost:5672/influxdb"]
-
-  ## Maximum messages to send over a connection.  Once this is reached, the
-  ## connection is closed and a new connection is made.  This can be helpful for
-  ## load balancing when not using a dedicated load balancer.
-  # max_messages = 0
-
-  ## Exchange to declare and publish to.
-  exchange = "telegraf"
-
-  ## Exchange type; common types are "direct", "fanout", "topic", "header", "x-consistent-hash".
-  # exchange_type = "topic"
-
-  ## If true, exchange will be passively declared.
-  # exchange_passive = false
-
-  ## Exchange durability can be either "transient" or "durable".
-  # exchange_durability = "durable"
-
-  ## Additional exchange arguments.
-  # exchange_arguments = { }
-  # exchange_arguments = {"hash_property" = "timestamp"}
-
-  ## Authentication credentials for the PLAIN auth_method.
-  # username = ""
-  # password = ""
-
-  ## Auth method. PLAIN and EXTERNAL are supported
-  ## Using EXTERNAL requires enabling the rabbitmq_auth_mechanism_ssl plugin as
-  ## described here: https://www.rabbitmq.com/plugins.html
-  # auth_method = "PLAIN"
-
-  ## Metric tag to use as a routing key.
-  ##   ie, if this tag exists, its value will be used as the routing key
-  # routing_tag = "host"
-
-  ## Static routing key.  Used when no routing_tag is set or as a fallback
-  ## when the tag specified in routing tag is not found.
-  # routing_key = ""
-  # routing_key = "telegraf"
-
-  ## Delivery Mode controls if a published message is persistent.
-  ##   One of "transient" or "persistent".
-  # delivery_mode = "transient"
-
-  ## InfluxDB database added as a message header.
-  ##   deprecated in 1.7; use the headers option
-  # database = "telegraf"
-
-  ## InfluxDB retention policy added as a message header
-  ##   deprecated in 1.7; use the headers option
-  # retention_policy = "default"
-
-  ## Static headers added to each published message.
-  # headers = { }
-  # headers = {"database" = "telegraf", "retention_policy" = "default"}
-
-  ## Connection timeout.  If not provided, will default to 5s.  0s means no
-  ## timeout (not recommended).
-  # timeout = "5s"
-
-  ## Optional TLS Config
-  # tls_ca = "/etc/telegraf/ca.pem"
-  # tls_cert = "/etc/telegraf/cert.pem"
-  # tls_key = "/etc/telegraf/key.pem"
-  ## Use TLS but skip chain & host verification
-  # insecure_skip_verify = false
-
-  ## If true use batch serialization format instead of line based delimiting.
-  ## Only applies to data formats which are not line based such as JSON.
-  ## Recommended to set to true.
-  # use_batch_format = false
-
-  ## Content encoding for message payloads, can be set to "gzip" to or
-  ## "identity" to apply no encoding.
-  ##
-  ## Please note that when use_batch_format = false each amqp message contains only
-  ## a single metric, it is recommended to use compression with batch format
-  ## for best results.
-  # content_encoding = "identity"
-
-  ## Data format to output.
-  ## Each data format has its own unique set of configuration options, read
-  ## more about them here:
-  ## https://github.com/influxdata/telegraf/blob/master/docs/DATA_FORMATS_OUTPUT.md
-  # data_format = "influx"
-`
-
-func (q *AMQP) SampleConfig() string {
+func (*AMQP) SampleConfig() string {
 	return sampleConfig
 }
 
-func (q *AMQP) Description() string {
-	return "Publishes metrics to an AMQP broker"
-}
-
 func (q *AMQP) SetSerializer(serializer serializers.Serializer) {
 	q.serializer = serializer
 }
@@ -259,7 +165,7 @@ func (q *AMQP) Write(metrics []telegraf.Metric) error {
 				if err != nil {
 					return err
 				}
-			} else {
+			} else if q.client != nil {
 				if err := q.client.Close(); err != nil {
 					q.Log.Errorf("Closing connection failed: %v", err)
 				}
