@@ -1,9 +1,11 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package cratedb
 
 import (
 	"context"
 	"crypto/sha512"
 	"database/sql"
+	_ "embed"
 	"encoding/binary"
 	"fmt"
 	"sort"
@@ -11,40 +13,38 @@ import (
 	"strings"
 	"time"
 
+	_ "github.com/jackc/pgx/v4/stdlib" //to register stdlib from PostgreSQL Driver and Toolkit
+
 	"github.com/influxdata/telegraf"
 	"github.com/influxdata/telegraf/config"
 	"github.com/influxdata/telegraf/plugins/outputs"
-	_ "github.com/jackc/pgx/v4/stdlib" //to register stdlib from PostgreSQL Driver and Toolkit
 )
 
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//go:embed sample.conf
+var sampleConfig string
+
 const MaxInt64 = int64(^uint64(0) >> 1)
 
 type CrateDB struct {
-	URL         string
-	Timeout     config.Duration
-	Table       string
-	TableCreate bool `toml:"table_create"`
-	DB          *sql.DB
+	URL          string
+	Timeout      config.Duration
+	Table        string
+	TableCreate  bool   `toml:"table_create"`
+	KeySeparator string `toml:"key_separator"`
+	DB           *sql.DB
 }
 
-var sampleConfig = `
-  # A github.com/jackc/pgx/v4 connection string.
-  # See https://pkg.go.dev/github.com/jackc/pgx/v4#ParseConfig
-  url = "postgres://user:password@localhost/schema?sslmode=disable"
-  # Timeout for all CrateDB queries.
-  timeout = "5s"
-  # Name of the table to store metrics in.
-  table = "metrics"
-  # If true, and the metrics table does not exist, create it automatically.
-  table_create = true
-`
+func (*CrateDB) SampleConfig() string {
+	return sampleConfig
+}
 
 func (c *CrateDB) Connect() error {
 	db, err := sql.Open("pgx", c.URL)
 	if err != nil {
 		return err
 	} else if c.TableCreate {
-		sql := `
+		query := `
 CREATE TABLE IF NOT EXISTS ` + c.Table + ` (
 	"hash_id" LONG INDEX OFF,
 	"timestamp" TIMESTAMP,
@@ -57,7 +57,7 @@ CREATE TABLE IF NOT EXISTS ` + c.Table + ` (
 `
 		ctx, cancel := context.WithTimeout(context.Background(), time.Duration(c.Timeout))
 		defer cancel()
-		if _, err := db.ExecContext(ctx, sql); err != nil {
+		if _, err := db.ExecContext(ctx, query); err != nil {
 			return err
 		}
 	}
@@ -68,15 +68,21 @@ CREATE TABLE IF NOT EXISTS ` + c.Table + ` (
 func (c *CrateDB) Write(metrics []telegraf.Metric) error {
 	ctx, cancel := context.WithTimeout(context.Background(), time.Duration(c.Timeout))
 	defer cancel()
-	if sql, err := insertSQL(c.Table, metrics); err != nil {
-		return err
-	} else if _, err := c.DB.ExecContext(ctx, sql); err != nil {
+
+	generatedSQL, err := insertSQL(c.Table, c.KeySeparator, metrics)
+	if err != nil {
 		return err
 	}
+
+	_, err = c.DB.ExecContext(ctx, generatedSQL)
+	if err != nil {
+		return err
+	}
+
 	return nil
 }
 
-func insertSQL(table string, metrics []telegraf.Metric) (string, error) {
+func insertSQL(table string, keyReplacement string, metrics []telegraf.Metric) (string, error) {
 	rows := make([]string, len(metrics))
 	for i, m := range metrics {
 		cols := []interface{}{
@@ -89,7 +95,7 @@ func insertSQL(table string, metrics []telegraf.Metric) (string, error) {
 
 		escapedCols := make([]string, len(cols))
 		for i, col := range cols {
-			escaped, err := escapeValue(col)
+			escaped, err := escapeValue(col, keyReplacement)
 			if err != nil {
 				return "", err
 			}
@@ -97,10 +103,10 @@ func insertSQL(table string, metrics []telegraf.Metric) (string, error) {
 		}
 		rows[i] = `(` + strings.Join(escapedCols, ", ") + `)`
 	}
-	sql := `INSERT INTO ` + table + ` ("hash_id", "timestamp", "name", "tags", "fields")
+	query := `INSERT INTO ` + table + ` ("hash_id", "timestamp", "name", "tags", "fields")
 VALUES
 ` + strings.Join(rows, " ,\n") + `;`
-	return sql, nil
+	return query, nil
 }
 
 // escapeValue returns a string version of val that is suitable for being used
@@ -113,7 +119,7 @@ VALUES
 // inputs.
 //
 // [1] https://github.com/influxdata/telegraf/pull/3210#issuecomment-339273371
-func escapeValue(val interface{}) (string, error) {
+func escapeValue(val interface{}, keyReplacement string) (string, error) {
 	switch t := val.(type) {
 	case string:
 		return escapeString(t, `'`), nil
@@ -131,11 +137,11 @@ func escapeValue(val interface{}) (string, error) {
 		return strconv.FormatBool(t), nil
 	case time.Time:
 		// see https://crate.io/docs/crate/reference/sql/data_types.html#timestamp
-		return escapeValue(t.Format("2006-01-02T15:04:05.999-0700"))
+		return escapeValue(t.Format("2006-01-02T15:04:05.999-0700"), keyReplacement)
 	case map[string]string:
-		return escapeObject(convertMap(t))
+		return escapeObject(convertMap(t), keyReplacement)
 	case map[string]interface{}:
-		return escapeObject(t)
+		return escapeObject(t, keyReplacement)
 	default:
 		// This might be panic worthy under normal circumstances, but it's probably
 		// better to not shut down the entire telegraf process because of one
@@ -154,7 +160,7 @@ func convertMap(m map[string]string) map[string]interface{} {
 	return c
 }
 
-func escapeObject(m map[string]interface{}) (string, error) {
+func escapeObject(m map[string]interface{}, keyReplacement string) (string, error) {
 	// There is a decent chance that the implementation below doesn't catch all
 	// edge cases, but it's hard to tell since the format seems to be a bit
 	// underspecified.
@@ -171,12 +177,15 @@ func escapeObject(m map[string]interface{}) (string, error) {
 	// Now we build our key = val pairs
 	pairs := make([]string, 0, len(m))
 	for _, k := range keys {
-		// escape the value of our key k (potentially recursive)
-		val, err := escapeValue(m[k])
+		key := escapeString(strings.ReplaceAll(k, ".", keyReplacement), `"`)
+
+		// escape the value of the value at k (potentially recursive)
+		val, err := escapeValue(m[k], keyReplacement)
 		if err != nil {
 			return "", err
 		}
-		pairs = append(pairs, escapeString(k, `"`)+" = "+val)
+
+		pairs = append(pairs, key+" = "+val)
 	}
 	return `{` + strings.Join(pairs, ", ") + `}`, nil
 }
@@ -184,7 +193,7 @@ func escapeObject(m map[string]interface{}) (string, error) {
 // escapeString wraps s in the given quote string and replaces all occurrences
 // of it inside of s with a double quote.
 func escapeString(s string, quote string) string {
-	return quote + strings.Replace(s, quote, quote+quote, -1) + quote
+	return quote + strings.ReplaceAll(s, quote, quote+quote) + quote
 }
 
 // hashID returns a cryptographic hash int64 hash that includes the metric name
@@ -194,7 +203,7 @@ func escapeString(s string, quote string) string {
 // [1] https://github.com/influxdata/telegraf/pull/3210#discussion_r148411201
 func hashID(m telegraf.Metric) int64 {
 	h := sha512.New()
-	h.Write([]byte(m.Name()))
+	h.Write([]byte(m.Name())) //nolint:revive // from hash.go: "It never returns an error"
 	tags := m.Tags()
 	tmp := make([]string, len(tags))
 	i := 0
@@ -205,7 +214,7 @@ func hashID(m telegraf.Metric) int64 {
 	sort.Strings(tmp)
 
 	for _, s := range tmp {
-		h.Write([]byte(s))
+		h.Write([]byte(s)) //nolint:revive // from hash.go: "It never returns an error"
 	}
 	sum := h.Sum(nil)
 
@@ -218,14 +227,6 @@ func hashID(m telegraf.Metric) int64 {
 	return int64(binary.LittleEndian.Uint64(sum))
 }
 
-func (c *CrateDB) SampleConfig() string {
-	return sampleConfig
-}
-
-func (c *CrateDB) Description() string {
-	return "Configuration for CrateDB to send metrics to."
-}
-
 func (c *CrateDB) Close() error {
 	return c.DB.Close()
 }
